@@ -127,18 +127,23 @@ copy_xattr (const char *from, const char *to)
 
 	  if (lsetxattr (to, name, value, size, 0) != 0)
 	    {
+#if defined(WITH_SELINUX)
 	      if (strcmp (name, "security.selinux") == 0 &&
 		  is_selinux_enabled() == 0)
 		fprintf (stderr,
 			 _("SELinux not enabled, ignore attribute %s for `%s'.\n"),
 			 name, to);
+
 	      else
-		fprintf (stderr, _("Cannot set attribute %s for `%s': %m\n"),
-			 name, to);
-	      if (value)
-		free (value);
-	      free (names);
-	      return -1;
+#endif
+		{
+		  fprintf (stderr, _("Cannot set attribute %s for `%s': %m\n"),
+			   name, to);
+		  if (value)
+		    free (value);
+		  free (names);
+		  return -1;
+		}
 	    }
 	}
       if (value)
