@@ -27,6 +27,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
+#include <signal.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <arpa/inet.h>
@@ -36,6 +37,10 @@
 #include "use_slp.h"
 #include "dbg_log.h"
 
+/*  This is the minimum we'll use, irrespective of config setting.
+    definately don't set to less than about 30 seconds.  */
+#define SLP_MIN_TIMEOUT 120
+
 static void
 pwdutilsSLPRegReport (SLPHandle hslp __attribute__ ((unused)),
 		      SLPError errcode, void* cookie)
@@ -45,35 +50,60 @@ pwdutilsSLPRegReport (SLPHandle hslp __attribute__ ((unused)),
 }
 
 /* the URL we use to register.  */
-
 static char *url = NULL;
 
+static char hostname[1024];
+static char *hname;
+static struct hostent *hp = NULL;
+static int saved_port;
+static int saved_timeout;
+
+static void
+do_refresh (int sig __attribute__ ((unused)))
+{
+  if (debug_level)
+    dbg_log ("Service registration almost expired, refreshing it");
+  register_slp (saved_port, saved_timeout);
+}
+
 int
-register_slp (int port)
+register_slp (int port, int slp_timeout)
 {
   SLPError err;
   SLPError callbackerr;
   SLPHandle hslp;
-  char hostname[1024];
-  char *hname;
-  struct hostent *hp = NULL;
+  int timeout;
+
+  saved_port = port;
+  saved_timeout = slp_timeout;
 
   if (url != NULL)
     {
-      dbg_log ("URL already registerd!");
-      return -1;
-    }
-
-  gethostname (hostname, sizeof (hostname));
-  if (isdigit (hostname[0]))
-    {
-      char addr[INADDRSZ];
-      if (inet_pton (AF_INET, hostname, &addr))
-        hp = gethostbyaddr (addr, sizeof (addr), AF_INET);
+      free (url);
+      url = NULL;
     }
   else
-    hp = gethostbyname (hostname);
-  hname = hp->h_name;
+    {
+      gethostname (hostname, sizeof (hostname));
+      if (isdigit (hostname[0]))
+	{
+	  char addr[INADDRSZ];
+	  if (inet_pton (AF_INET, hostname, &addr))
+	    hp = gethostbyaddr (addr, sizeof (addr), AF_INET);
+	}
+      else
+	hp = gethostbyname (hostname);
+      hname = hp->h_name;
+    }
+
+  if (slp_timeout == 0)
+    timeout = SLP_LIFETIME_MAXIMUM; /* don't expire, ever */
+  else if (SLP_MIN_TIMEOUT > slp_timeout)
+    timeout = SLP_MIN_TIMEOUT; /* use a reasonable minimum */
+  else if (SLP_LIFETIME_MAXIMUM <= slp_timeout)
+    timeout = (SLP_LIFETIME_MAXIMUM - 1); /* as long as possible */
+  else
+    timeout = slp_timeout;
 
   if (asprintf (&url, "service:rpasswdd://%s:%i/", hname, port) < 0)
     {
@@ -116,6 +146,18 @@ register_slp (int port)
   /* Now that we're done using slp, close the slp handle */
   SLPClose (hslp);
 
+  /* Set up a timer to refresh the service records */
+  if (timeout != SLP_LIFETIME_MAXIMUM)
+    {
+      struct sigaction act;
+
+      act.sa_handler = do_refresh;
+      if (sigaction (SIGALRM, &act, NULL) != 0)
+	dbg_log ("SLP: error establishing signal handler\n");
+
+      alarm (timeout - 15);
+    }
+
   return 0;
 }
 
@@ -139,12 +181,16 @@ deregister_slp ()
       return -1;
     }
 
-    /* DeRegister a service with SLP */
+  /* Disable possibel alarm call.  */
+  alarm (0);
+
+  /* DeRegister a service with SLP */
   err = SLPDereg (hslp, url, pwdutilsSLPRegReport, &callbackerr);
 
   free (url);
   url = NULL;
 
+
   /* err may contain an error code that occurred as the slp library    */
   /* _prepared_ to make the call.                                     */
   if ((err != SLP_OK) || (callbackerr != SLP_OK))
