@@ -17,8 +17,7 @@
 #ifndef __USE_SLP_H__
 #define __USE_SLP_H__
 
-int register_slp (int port);
+int register_slp (int port, int timeout);
 int deregister_slp (void);
 
 #endif
-
