@@ -57,8 +57,8 @@
 static void
 print_usage (FILE *stream, const char *program)
 {
-  fprintf (stream, _("Usage: %s [-l] [group]\n"),
-           program);
+  fprintf (stream, _("Usage: %s [-l|-c command] [group]\n"),
+	   program);
 }
 
 static void
@@ -70,6 +70,7 @@ print_help (const char *program)
 
   fputs (_("  -l, --login    reinitialize environment as if logged in\n"),
 	 stdout);
+  fputs (_("  -c  command    Execute `command' with new group\n"), stdout);
   fputs (_("      --help     Give this help list\n"), stdout);
   fputs (_("  -u, --usage    Give a short usage message\n"), stdout);
   fputs (_("  -v, --version  Print program version\n"), stdout);
@@ -81,6 +82,7 @@ main (int argc, char *argv[])
   struct passwd *pw;
   char *program;
   char *cp, *shell;
+  char *c_flag = NULL;
   int l_flag = 0;
   int ngroups_max = MIN (sysconf (_SC_NGROUPS_MAX), INT_MAX);
 
@@ -99,13 +101,14 @@ main (int argc, char *argv[])
       int option_index = 0;
       static struct option long_options[] = {
         {"login",   no_argument, NULL, 'l' },
+	{"command", required_argument, NULL, 'c'},
         {"version", no_argument, NULL, 'v' },
         {"usage",   no_argument, NULL, 'u' },
         {"help",    no_argument, NULL, '\255' },
         {NULL,      0,           NULL, '\0'}
       };
 
-      c = getopt_long (argc, argv, "lvu",
+      c = getopt_long (argc, argv, "lvuc:",
                        long_options, &option_index);
       if (c == (-1))
         break;
@@ -115,6 +118,9 @@ main (int argc, char *argv[])
 	case 'l':
 	  l_flag = 1;
 	  break;
+	case 'c':
+	  c_flag = optarg;
+	  break;
 	case '\255':
 	  print_help (program);
 	  return 0;
@@ -133,12 +139,31 @@ main (int argc, char *argv[])
   argc -= optind;
   argv += optind;
 
+  /* Special hack for compatibility with "sg group command"  */
+  if (argc == 2 && strcmp (program, "sg") == 0)
+    {
+      c_flag = argv[1];
+      argc--;
+    }
+
   if (argc > 1)
     {
       fprintf (stderr, _("%s: Too many arguments\n"), program);
       print_error (program);
       return E_USAGE;
     }
+  else if (c_flag && l_flag)
+    {
+      fprintf (stderr, _("%s: -l and -c are exclusive\n"), program);
+      print_error (program);
+      return E_USAGE;
+    }
+  else if (c_flag && argc != 1)
+    {
+      fprintf (stderr, _("%s: -c requires a group argument\n"), program);
+      print_error (program);
+      return E_USAGE;
+    }
 
   if ((pw = getpwuid (getuid())) == NULL)
     {
@@ -253,11 +278,14 @@ main (int argc, char *argv[])
 	if (grp->gr_gid == grouplist[i])
 	  break;
 
-      if (i == ngroups && ngroups < ngroups_allocated && ngroups < ngroups_max)
-	grouplist[ngroups++] = grp->gr_gid;
-      else
-	fprintf (stderr, _("%s: too many groups, not added.\n"),
-		 program);
+      if (i == ngroups)
+	{
+	  if (ngroups < ngroups_allocated && ngroups < ngroups_max)
+	    grouplist[ngroups++] = grp->gr_gid;
+	  else
+	    fprintf (stderr, _("%s: too many groups, not added.\n"),
+		     program);
+	}
 #endif
 
       if (setgroups (ngroups, (grouplist)) < 0)
@@ -336,6 +364,8 @@ main (int argc, char *argv[])
 
       execvp (shell, args);
     }
+  else if (c_flag)
+    execl ("/bin/sh", "sh", "-c", c_flag, (char *) 0);
   else
     execl (shell, basename (shell), NULL);
 
