@@ -97,7 +97,7 @@ static void
 print_usage (FILE *stream, const char *program)
 {
 #ifdef USE_SLP
-  fprintf (stream, _("Usage: %s [-4] [-6] [-d] [-c certificate] [-k privatekey] [-p port] [--slp]\n"),
+  fprintf (stream, _("Usage: %s [-4] [-6] [-d] [-c certificate] [-k privatekey] [-p port]\n       [--slp [--slp-timeout timeout]]\n"),
 	   program);
 #else
   fprintf (stream, _("Usage: %s [-4] [-6] [-d] [-c certificate] [-k privatekey] [-p port]\n"),
@@ -121,6 +121,7 @@ print_help (const char *program)
 	 stdout);
 #ifdef USE_SLP
   fputs (_("  --slp          Register at local SLP server\n"), stdout);
+  fputs (_("  --slp-timeout  Set timeout for re-registration\n"), stdout);
 #endif
   fputs (_("  --help         Give this help list\n"), stdout);
   fputs (_("  --usage        Give a short usage message\n"), stdout);
@@ -1018,6 +1019,7 @@ main (int argc, char **argv)
   char *certificate = "/etc/rpasswdd.pem";
   char *privatekey = "/etc/rpasswdd.pem";
   int port = -1;
+  int slp_timeout = 3600;
   SSL_METHOD *meth;
 
   /* Set locale via LC_ALL.  */
@@ -1038,6 +1040,7 @@ main (int argc, char **argv)
 	  {"ipv6",     no_argument,       NULL, '6'},
 	  {"certificate", required_argument, NULL, 'c'},
 	  {"privatekey",  required_argument, NULL, 'k'},
+	  {"slp-timeout", required_argument, NULL, '\251'},
 	  {"slp",      no_argument,       NULL, '\252'},
           {"version",  no_argument,       NULL, '\255'},
           {"usage",    no_argument,       NULL, '\254'},
@@ -1071,6 +1074,9 @@ main (int argc, char **argv)
 	  port = htons (atol (optarg));
 	  break;
 #ifdef USE_SLP
+	case '\251':
+	  slp_timeout = atol (optarg);
+	  break;
 	case '\252':
 	  use_slp = 1;
 	  break;
@@ -1201,7 +1207,7 @@ main (int argc, char **argv)
 #ifdef USE_SLP
   /* Register at local SLP server.  */
   if (use_slp)
-    register_slp (ntohs(port));
+    register_slp (ntohs(port), slp_timeout);
 #endif
 
   /* Handle incoming requests.  */
